<?php namespace Notes\Route;

use Notes\ObjectReflection,
    Notes\ObjectResolver;

use RuntimeException, DirectoryIterator, Generator, Closure;

class RouteFetcher {

    protected array $folderList;

    protected Closure $callback;

    public array $defaultMethods = [ 'GET', 'POST' ];

    protected array $annotations;

    public function __construct(?Closure $callback = null, ?array $folderList = null, ?array $annotations = null)
    {
        if ($callback !== null) {
            $this->callback = $callback;
        }

        if ($folderList !== null) {
            $this->folderList = $folderList;
        }

        if ($annotations !== null) {
            $this->annotations = $annotations;
        }
        else {
            $this->annotations = [
                'object' => Annotation\Object\Route::class,
                'method' => Annotation\Method\Route::class,
            ];
        }
    }

    public function addFolder($folder) : void
    {
        $this->folderList[] = $folder;
    }

    public function setFolderList(array $list) : void
    {
        $this->folderList = $list;
    }

    public function scan() : Generator
    {
        foreach($this->folderList as $namespace => $folder) {
            if ( ! file_exists($folder) ) {
                throw new RuntimeException(sprintf("Folder `%s` can not be found or scanned", $folder));
            }

            foreach (new DirectoryIterator($folder) as $fileinfo) {
                if ( ! $fileinfo->isDot() ) {
                    yield $namespace => $fileinfo;
                }
            }
        }
    }

    public function compile() : Generator
    {
        foreach($this->scan() as $namespace => $file) {
            $class = $this->generateClassname($file->getBasename(".php"), $namespace);
            $methods = $this->defaultMethods;

            # Should generate an equivalent of Ulmus's object reflection here !
            $objectResolver = new ObjectResolver($class, true, true, false, true);

            if ( null !== ( $object = $objectResolver->getAnnotationFromClassname( $this->annotations['object'] ) ) ) {
                if ( $object->methods ?? false ) {
                    $methods = $object->methods;
                }
            }

            $routeList = $objectResolver->getAnnotationListFromClassname( $this->annotations['method'] );

            foreach($routeList as $func => $route) {
                $route->class = $class;
                $route->classMethod = $func;

                if ( false === ( $route->methods ?? false ) ) {
                    $route->methods = $methods;
                }

                if ( false !== ( $this->callback ?? false ) ) {
                    call_user_func_array($this->callback, [ $route ]);
                }

                yield $route;
            }
        }
    }

    protected function generateClassname($file, $namespace)
    {
        return "\\$namespace\\$file";
    }
}
